"""Register custom fields to time logs when they are created and updated.

In this module you can register custom fields to a time log when it gets
created and when it gets updated. You can add any data to a time log. As an
example, here you can keep record about the Nuke environment, certain
environment variables of your current context, statistics about your working
file and much more.

A time log's data structure is of dict type in the following format:

    {
        "comment": "",
        "custom": {},
        "id": "690042e6-142e-4784-91bb-7adb85825317",
        "path": "/mnt/projects/show-a/001/001/show-a_001_001_comp_v001.nk",
        "tags": [],
        "time": 75,
        "timestamp": 1584226841,
        "user": "John Doe"
    }

Note the 'custom' key in here. You can inject custom data into it. The custom
key itself is a dict which means you can add multiple keys to it as you need.
For that, simply create a function that produces any data and register it as
described below. Keep in mind that your function's return value must be
serializable, otherwise you won't be able to save it to the time log.

In order to add data to the custom key, we provide two different functions:

osl.register_custom_field_on_creation: Use this function to add custom data
to the 'custom' key when the time log gets created. That happens the first time
you save your working file. The data gets only added once and will not change.

register_custom_field_on_update: Use this function to add custom data to the
'custom' key each time the time log gets updated. The custom data will be
updated as well. The time logging interval is customizable. You can adjust it
in smartLog's settings in the 'Log interval' widget. Each time it times out,
the custom data will be updated as well. Due to the nature of triggering your
functions with each timeout, we recommend you keep your functions light weight.

register_custom_field_on_creation and register_custom_field_on_update add both
to the 'custom' dict. Both functions take two arguments: The first argument is
the name of the key to store the value to. The second argument is the callable
to execute. Keep in mind to use a callable and not calling the function
directly, meaning don't use round brackets as this would execute the function
in place which is not what you want.

Example:
Let's store the used Nuke version that the working file was created and store
it inside the 'custom' key. For that we want to add this information just once,
so we will register it using 'register_custom_field_on_creation'.

The function to get the Nuke version is as follows:

>>> import nuke

>>> def get_nuke_version():
...     return nuke.env["NukeVersionString"]

We can now register it using the following line of code:

>>> osl.register_custom_field_on_creation("nuke_version", get_nuke_version)

This will add a custom 'nuke_version' key inside the 'custom' dict when
creating a new time log, meaning the first time you save a new working file.
Again, keep in mind to provide a callable as second argument when registering
the custom data. Don't call the function in place using round brackets.

The time log will now look something like this:

    {
        "comment": "",
        "custom": {
            "nuke_version": "12.1v1",
        },
        "id": "690042e6-142e-4784-91bb-7adb85825317",
        "path": "/mnt/projects/show-a/001/001/show-a_001_001_comp_v001.nk",
        "tags": [],
        "time": 75,
        "timestamp": 1584226841,
        "user": "John Doe"
    }

Sometimes however, you want to continuously update your data whenever the
time log gets updated. As an example, you can add a callback that gives you
statistics about the working file. As an example, have a look at the function
'get_working_file_stats'. This function gets you information about the number
of Read- and Write nodes in your working file as well as the dependencies being
read by all enabled Read nodes that contain file paths. As mentioned above,
keep in mind to keep the functions light weight as they will be triggered
whenever your time log gets updated.

"""

# Import built-in modules
import os
import platform

# Import third-party modules
import nuke

# Import local modules
from smartLog import osl


def get_environment():
    """Get specific information of the current environment.

    Notes:
        This is an example function to demonstrate how to inject custom data
        into time logs. We add this data just once by registering it below
        using 'register_custom_field_on_creation'. Keep in mind that you can
        add anything you like and need in here. As an example, you could add
        the whole nuke.env data into here, other custom environment variables
        of your studio's pipeline, etc.

    Returns:
        dict: Specific information of the current environment.

    """
    return {
        "ocio": os.environ.get("OCIO"),
        "system": platform.system(),
        "nuke_version": nuke.env["NukeVersionString"]
    }


def get_working_file_stats():
    """Get some information about the current state of the working file.

    Notes:
        This is an example function to demonstrate how to inject custom data
        into time logs. We add and update this data with every update by
        registering it below using 'register_custom_field_on_update'. Keep in
        mind that you can add anything you like and need in here. In order to
        avoid performance hits, we recommend to keep update callbacks light
        weight.

    Returns:
        dict: Information about the current state of the working file in the
            format:
            {
                "write": <number of Write nodes in working file>,
                "read": <number of Read nodes in working file>,
                "read dependencies": <List of used dependencies of all Reads>
            }

    """
    def count_nodes_for_class(class_name):
        """Count enabled nodes recursively for the given node class.

        Args:
            class_name (str): The name of the node class to count existence.
                Counts recursively, meaning withing groups and nested groups.

        Returns:
            int: The number of enabled occurrences of the given class in the
                working file, including nested occurrences.

        """
        nodes = [
            node for node in nuke.allNodes(recurseGroups=True)
            if node.Class() == class_name
            and not node["disable"].value()
        ]
        return len(nodes)

    def get_read_dependencies():
        """Get all used dependencies of all enabled Read nodes reading files.

        Returns:
            :obj:`list` of :obj:`str`: Absolute paths of all used dependencies
                of all enabled Read nodes that reference files.

        """
        reads = [
            node for node in nuke.allNodes(recurseGroups=True)
            if node.Class() == "Read"
            and not node["disable"].value()
            and node["file"].value()
        ]

        dependencies = [
            node["file"].evaluate()
            for node in reads
        ]

        return sorted(dependencies)

    return {
        "write": count_nodes_for_class("Write"),
        "read": count_nodes_for_class("Read"),
        "read dependencies": get_read_dependencies()
    }


# Register custom fields -once- when creating a new time log. This uses
# the function call 'register_custom_field_on_creation'.
# You can register multiple custom fields in here.
# osl.register_custom_field_on_creation("environ", get_environment)
# osl.register_custom_field_on_creation("something", some_callable)
# ...

# Register custom fields on every update. This uses the function call
# 'register_custom_field_on_update'. It is recommended to keep the
# callables light weight as they will be called with each time log update.
# The update interval is configurable in smartLog's setting's 'Log interval'
# widget. You can register multiple custom fields in here.
# osl.register_custom_field_on_update("stats", get_working_file_stats)
# osl.register_custom_field_on_update("something else", some_other_callable)
# ...
