"""Create custom commands that processes on selected time logs.

This module exists so that you can add custom functionality that gets triggered
on selected logs. You can basically do anything you you like here. Simply
create a function with one argument ``selected_logs`. This argument will get
the data of your selected logs automatically assigned when it is triggered
via smartLog's right click menu on selected time logs in history viewing mode.
A selection of logs uses the data type of a list of dicts in the following
format:
    [
        {
            'comment': 'Some comment',
            'tags': ['prep', 'WIP'],
            'timestamp': 1578084114,
            'user': 'James Miller',
            'time': 3840,
            'path': '/path/to/project/tzw/tzw_010_040_comp-prep_v008_jmi.nk',
            'log_path': '/path/to/logs/1577865467.json',
            'id': '90e60342-3d6a-4c91-80d7-7303b996d9df'
        },
        {
            'comment': '',
            'tags': [],
            'timestamp': 1577865467,
            'user': 'Steve Fisher',
            'time': 9300,
            'path': '/path/to/project/mnt/mnt_300_020_comp_v027_sfi.nk',
            'log_path': '/path/to/logs/1577865498.json',
            'id': '0e811744-b3c8-4259-ae56-d20f5b86710b'
        },
        ...
    ]

That means, when executing your custom commands, the ``selected_logs`` will
contain the data of your selected logs in a list as seen above. You are free
to do whatever you need to do with this data.

As an example, we provide two custom commands as seen below:

- sticky_from_timelogs: This custom command will create StickyNotes in the DAG
for all selected logs and will populate the StickyNote's label with data from
the logs.

- import_script_into_dag: This custom command will insert the nodes of the
working files of the selected logs into the DAG.

Keep in mind that you are free to do anything needed here and the custom
functions are not restricted to process only inside Nuke. For instance, you
could communicate with your database, perform certain actions on your file
system and do many many more things here.

Once you have created a custom function, register it to smartLog using the
register command as seen below:

osl.register_custom_command("Name of the command", my_custom_callable)

See below for an example.

"""

# Import built-in modules
import datetime

# Import third-party modules
import nuke

# Import local modules
from smartLog import osl


def sticky_from_timelogs(selected_logs):
    """Create StickyNotes containing data of the given selected logs.

    Args:
        selected_logs (:obj:`list` of :obj:`dict`): Sequence of selected time
            logs in the format as shown above.

    """
    content = (
        "User: {user}<br />"
        "Comment_ {comment}<br />"
        "Time: {time}<br />"
    )

    for log in selected_logs:
        sticky = nuke.createNode("StickyNote")

        # The time gets stored in seconds. Convert the time to a human
        # readable format.
        log["time"] = str(datetime.timedelta(seconds=(log["time"])))

        content = content.format(**log)
        sticky["label"].setValue(content)


def import_script_into_dag(selected_logs):
    """Import the working files of the selected logs into the DAG.

    Args:
        selected_logs (:obj:`list` of :obj:`dict`): Sequence of selected time
            logs in the format as shown above.

    """
    for log in selected_logs:
        nuke.nodePaste(log["path"])


# Register custom commands to smartLog.
#
# The first argument is the label of the command. The second argument is the
# callable to invoke. It will automatically get the data of the selected logs
# assigned in the format as described above.
# Enable one or both lines below to get the custom commands into smartLog's
# 'custom' sub menu of the right click menu when right clicking time logs in
# history mode.
#
# osl.register_custom_command("Sticky from TimeLogs", sticky_from_timelogs)
# osl.register_custom_command("Import Script into DAG", import_script_into_dag)
